@@ -1,7 +1,7 @@
 %%
 %% %CopyrightBegin%
 %%
-%% Copyright Ericsson AB 1996-2017. All Rights Reserved.
+%% Copyright Ericsson AB 1996-2021. All Rights Reserved.
 %%
 %% Licensed under the Apache License, Version 2.0 (the "License");
 %% you may not use this file except in compliance with the License.
@@ -39,6 +39,7 @@
 %% We processes the load request queue as a "background" job..
 
 -module(mnesia_controller).
+-compile([{nowarn_deprecated_function, [{erlang,phash,2}]}]).
 
 -behaviour(gen_server).
 
@@ -80,7 +81,7 @@
 	 wait_for_tables/2,
 	 get_network_copy/3,
 	 merge_schema/0,
-	 start_remote_sender/4,
+	 start_remote_sender/5,
 	 schedule_late_disc_load/2
 	]).
 
@@ -153,7 +154,8 @@ max_loaders() ->
 
 -record(send_table, {table,
 		     receiver_pid,
-		     remote_storage
+		     remote_storage,
+                     reason
 		    }).
 
 -record(disc_load, {table,
@@ -185,9 +187,10 @@ max_loaders() ->
 		      worker_res
 		     }).
 
+%% Local function in order to avoid external function call
 val(Var) ->
-    case ?catch_val(Var) of
-	{'EXIT', _} -> mnesia_lib:other_val(Var);
+    case ?catch_val_and_stack(Var) of
+	{'EXIT', Stacktrace} -> mnesia_lib:other_val(Var, Stacktrace);
 	Value -> Value
     end.
 
@@ -330,35 +333,39 @@ release_schema_commit_lock() ->
 
 %% Special for preparation of add table copy
 get_network_copy(Tid, Tab, Cs) ->
-%   We can't let the controller queue this one
-%   because that may cause a deadlock between schema_operations
-%   and initial tableloadings which both takes schema locks.
-%   But we have to get copier_done msgs when the other side
-%   goes down.
-    call({add_other, self()}),
-    Reason = {dumper,{add_table_copy, Tid}},
-    Work = #net_load{table = Tab,reason = Reason,cstruct = Cs},
-    %% I'll need this cause it's linked trough the subscriber
-    %% might be solved by using monitor in subscr instead.
-    process_flag(trap_exit, true),
-    Load = load_table_fun(Work),
-    Res = ?CATCH(Load()),
-    process_flag(trap_exit, false),
-    call({del_other, self()}),
-    case Res of
- 	#loader_done{is_loaded = true} ->
- 	    Tab = Res#loader_done.table_name,
- 	    case Res#loader_done.needs_announce of
- 		true ->
- 		    i_have_tab(Tab);
- 		false ->
- 		    ignore
- 	    end,
- 	    Res#loader_done.reply;
-	#loader_done{} ->
- 	    Res#loader_done.reply;
- 	Else ->
- 	    {not_loaded, Else}
+    %%   We can't let the controller queue this one
+    %%   because that may cause a deadlock between schema_operations
+    %%   and initial tableloadings which both takes schema locks.
+    %%   But we have to get copier_done msgs when the other side
+    %%   goes down.
+    case call({add_other, self()}) of
+        ok ->
+            Reason = {dumper,{add_table_copy, Tid}},
+            Work = #net_load{table = Tab,reason = Reason,cstruct = Cs},
+            %% I'll need this cause it's linked trough the subscriber
+            %% might be solved by using monitor in subscr instead.
+            process_flag(trap_exit, true),
+            Load = load_table_fun(Work),
+            Res = ?CATCH(Load()),
+            process_flag(trap_exit, false),
+            call({del_other, self()}),
+            case Res of
+                #loader_done{is_loaded = true} ->
+                    Tab = Res#loader_done.table_name,
+                    case Res#loader_done.needs_announce of
+                        true ->
+                            i_have_tab(Tab);
+                        false ->
+                            ignore
+                    end,
+                    Res#loader_done.reply;
+                #loader_done{} ->
+                    Res#loader_done.reply;
+                Else ->
+                    {not_loaded, Else}
+            end;
+        {error, Else} ->
+            {not_loaded, Else}
     end.
 
 %% This functions is invoked from the dumper
@@ -446,7 +453,7 @@ try_schedule_late_disc_load(Tabs, Reason, MsgTag) ->
 		    [BadNodes]),
 	    try_schedule_late_disc_load(Tabs, Reason, MsgTag);
 	{aborted, AbortReason} ->
-	    fatal("Cannot late_load_tables~p: ~p~n",
+	    fatal("Cannot late_load_tables ~tp: ~tp~n",
 		  [[Tabs, Reason, MsgTag], AbortReason])
     end.
 
@@ -455,8 +462,6 @@ connect_nodes(Ns) ->
 
 connect_nodes(Ns, UserFun) ->
     case mnesia:system_info(is_running) of
-	no ->
-	    {error, {node_not_running, node()}};
 	yes ->
 	    Pid = spawn_link(?MODULE,connect_nodes2,[self(),Ns, UserFun]),
 	    receive
@@ -473,7 +478,9 @@ connect_nodes(Ns, UserFun) ->
 		    end;
 		{'EXIT', Pid, Reason} ->
 		    {error, Reason}
-	    end
+	    end;
+        _ -> %% no, starting or stopping not ready to make a connection yet
+	    {error, {node_not_running, node()}}
     end.
 
 connect_nodes2(Father, Ns, UserFun) ->
@@ -535,7 +542,7 @@ try_merge_schema(Nodes, Told0, UserFun) ->
 		   end,
 	    try_merge_schema(Nodes, Told, UserFun);
 	{atomic, {"Cannot get cstructs", Node, Reason}} ->
-	    dbg_out("Cannot get cstructs, Node ~p ~p~n", [Node, Reason]),
+	    dbg_out("Cannot get cstructs, Node ~p ~tp~n", [Node, Reason]),
 	    timer:sleep(300), % Avoid a endless loop look alike
 	    try_merge_schema(Nodes, Told0, UserFun);
 	{aborted, {shutdown, _}} ->  %% One of the nodes is going down
@@ -771,21 +778,17 @@ handle_call({unannounce_add_table_copy, [Tab, Node], From}, ReplyTo, State) ->
 	    noreply(State#state{early_msgs = [{call, Msg, undefined} | Msgs]})
     end;
 
-handle_call({net_load, Tab, Cs}, From, State) ->
-    State2 =
-	case State#state.schema_is_merged of
-	    true ->
-		Worker = #net_load{table = Tab,
-				   opt_reply_to = From,
-				   reason = {dumper,{add_table_copy, unknown}},
-				   cstruct = Cs
-				  },
-		add_worker(Worker, State);
-	    false ->
-		reply(From, {not_loaded, schema_not_merged}),
-		State
-	end,
-    noreply(State2);
+handle_call({add_other, Who}, _From, State = #state{others=Others0, schema_is_merged=SM}) ->
+    case SM of
+        true ->
+            Others = [Who|Others0],
+            {reply, ok, State#state{others=Others}};
+        false ->
+            {reply, {error, {not_active,schema,node()}}, State}
+    end;
+handle_call({del_other, Who}, _From, State = #state{others=Others0}) ->
+    Others = lists:delete(Who, Others0),
+    {reply, ok, State#state{others=Others}};
 
 handle_call(Msg, From, State) when State#state.schema_is_merged /= true ->
     %% Buffer early messages
@@ -798,7 +801,7 @@ handle_call({late_disc_load, Tabs, Reason, RemoteLoaders}, From, State) ->
 
 handle_call({unblock_table, Tab}, _Dummy, State) ->
     Var = {Tab, where_to_commit},
-    case val(Var) of
+    case ?catch_val(Var) of
 	{blocked, List} ->
 	    set(Var, List); % where_to_commit
 	_ ->
@@ -818,20 +821,13 @@ handle_call({block_table, [Tab], From}, _Dummy, State) ->
 handle_call({check_w2r, _Node, Tab}, _From, State) ->
     {reply, val({Tab, where_to_read}), State};
 
-handle_call({add_other, Who}, _From, State = #state{others=Others0}) ->
-    Others = [Who|Others0],
-    {reply, ok, State#state{others=Others}};
-handle_call({del_other, Who}, _From, State = #state{others=Others0}) ->
-    Others = lists:delete(Who, Others0),
-    {reply, ok, State#state{others=Others}};
-
 handle_call(Msg, _From, State) ->
-    error("~p got unexpected call: ~p~n", [?SERVER_NAME, Msg]),
+    error("~p got unexpected call: ~tp~n", [?SERVER_NAME, Msg]),
     noreply(State).
 
 late_disc_load(TabsR, Reason, RemoteLoaders, From,
 	       State = #state{loader_queue = LQ, late_loader_queue = LLQ}) ->
-    verbose("Intend to load tables: ~p~n", [TabsR]),
+    verbose("Intend to load tables: ~tp~n", [TabsR]),
     ?eval_debug_fun({?MODULE, late_disc_load},
 		    [{tabs, TabsR},
 		     {reason, Reason},
@@ -1036,6 +1032,12 @@ handle_cast({disc_load, Tab, Reason}, State) ->
     State2 = add_worker(Worker, State),
     noreply(State2);
 
+handle_cast({send_table, Tab, Pid, Storage}, State) ->
+    %% {protocol, Node} = {8,5} or less
+    %% Let reason be undefined
+    Worker = #send_table{table=Tab, receiver_pid=Pid, remote_storage=Storage},
+    State2 = add_worker(Worker, State),
+    noreply(State2);
 handle_cast(Worker = #send_table{}, State) ->
     State2 = add_worker(Worker, State),
     noreply(State2);
@@ -1118,7 +1120,7 @@ handle_cast({adopt_orphans, Node, Tabs}, State) ->
     noreply(State2);
 
 handle_cast(Msg, State) ->
-    error("~p got unexpected cast: ~p~n", [?SERVER_NAME, Msg]),
+    error("~p got unexpected cast: ~tp~n", [?SERVER_NAME, Msg]),
     noreply(State).
 
 handle_sync_tabs([Tab | Tabs], From) ->
@@ -1166,7 +1168,7 @@ handle_info(#dumper_done{worker_pid=Pid, worker_res=Res}, State) ->
 	    State3 = opt_start_worker(State2),
 	    noreply(State3);
 	true ->
-	    fatal("Dumper failed: ~p~n state: ~p~n", [Res, State]),
+	    fatal("Dumper failed: ~p~n state: ~tp~n", [Res, State]),
 	    {stop, fatal, State}
     end;
 
@@ -1241,15 +1243,19 @@ handle_info(Done = #loader_done{worker_pid=WPid, table_name=Tab}, State0) ->
 handle_info(#sender_done{worker_pid=Pid, worker_res=Res}, State)  ->
     Senders = get_senders(State),
     {value, {Pid,_Worker}} = lists:keysearch(Pid, 1, Senders),
-    if
-	Res == ok ->
+    case Res of
+	ok ->
 	    State2 = State#state{sender_pid = lists:keydelete(Pid, 1, Senders)},
 	    State3 = opt_start_worker(State2),
 	    noreply(State3);
-	true ->
+        {error, {no_exists, _Tab}} ->
+            State2 = State#state{sender_pid = lists:keydelete(Pid, 1, Senders)},
+	    State3 = opt_start_worker(State2),
+	    noreply(State3);
+        _ ->
 	    %% No need to send any message to the table receiver
 	    %% since it will soon get a mnesia_down anyway
-	    fatal("Sender failed: ~p~n state: ~p~n", [Res, State]),
+	    fatal("Sender failed: ~p~n state: ~tp~n", [Res, State]),
 	    {stop, fatal, State}
     end;
 
@@ -1257,7 +1263,7 @@ handle_info({'EXIT', Pid, R}, State) when Pid == State#state.supervisor ->
     ?SAFE(set(mnesia_status, stopping)),
     case State#state.dumper_pid of
 	undefined ->
-	    dbg_out("~p was ~p~n", [?SERVER_NAME, R]),
+	    dbg_out("~p was ~tp~n", [?SERVER_NAME, R]),
 	    {stop, shutdown, State};
 	_ ->
 	    noreply(State#state{is_stopping = true})
@@ -1266,12 +1272,12 @@ handle_info({'EXIT', Pid, R}, State) when Pid == State#state.supervisor ->
 handle_info({'EXIT', Pid, R}, State) when Pid == State#state.dumper_pid ->
     case State#state.dumper_queue of
 	[#schema_commit_lock{}|Workers] -> %% Schema trans crashed or was killed
-	    dbg_out("WARNING: Dumper ~p exited ~p~n", [Pid, R]),
+	    dbg_out("WARNING: Dumper ~p exited ~tp~n", [Pid, R]),
 	    State2 = State#state{dumper_queue = Workers, dumper_pid = undefined},
 	    State3 = opt_start_worker(State2),
 	    noreply(State3);
 	_Other ->
-	    fatal("Dumper or schema commit crashed: ~p~n state: ~p~n", [R, State]),
+	    fatal("Dumper or schema commit crashed: ~p~n state: ~tp~n", [R, State]),
 	    {stop, fatal, State}
     end;
 
@@ -1280,15 +1286,15 @@ handle_info(Msg = {'EXIT', Pid, R}, State) when R /= wait_for_tables_timeout ->
 	true ->
 	    %% No need to send any message to the table receiver
 	    %% since it will soon get a mnesia_down anyway
-	    fatal("Sender crashed: ~p~n state: ~p~n", [{Pid,R}, State]),
+	    fatal("Sender crashed: ~p~n state: ~tp~n", [{Pid,R}, State]),
 	    {stop, fatal, State};
 	false ->
 	    case lists:keymember(Pid, 1, get_loaders(State)) of
 		true ->
-		    fatal("Loader crashed: ~p~n state: ~p~n", [R, State]),
+		    fatal("Loader crashed: ~p~n state: ~tp~n", [R, State]),
 		    {stop, fatal, State};
 		false ->
-		    error("~p got unexpected info: ~p~n", [?SERVER_NAME, Msg]),
+		    error("~p got unexpected info: ~tp~n", [?SERVER_NAME, Msg]),
 		    noreply(State)
 	    end
     end;
@@ -1308,7 +1314,7 @@ handle_info({'EXIT', Pid, wait_for_tables_timeout}, State) ->
     noreply(State);
 
 handle_info(Msg, State) ->
-    error("~p got unexpected info: ~p~n", [?SERVER_NAME, Msg]),
+    error("~p got unexpected info: ~tp~n", [?SERVER_NAME, Msg]),
     noreply(State).
 
 sync_tab_timeout(Pid, [{{sync_tab, Tab}, Pids} | Tail]) ->
@@ -1456,7 +1462,8 @@ orphan_tables([Tab | Tabs], Node, Ns, Local, Remote) ->
 				    L = [Tab | Local],
 				    orphan_tables(Tabs, Node, Ns, L, Remote);
 				Masters ->
-				    R = [{Tab, Masters} | Remote],
+                                    %% Do not disc_load table from RamCopyHolders
+				    R = [{Tab, Masters -- RamCopyHolders} | Remote],
 				    orphan_tables(Tabs, Node, Ns, Local, R)
 			    end;
 			_ ->
@@ -1585,7 +1592,14 @@ initial_safe_loads() ->
     end.
 
 last_consistent_replica(Tab, Downs) ->
-    Cs = val({Tab, cstruct}),
+    case ?catch_val({Tab, cstruct}) of
+        #cstruct{} = Cs ->
+            last_consistent_replica(Cs, Tab, Downs);
+        _ ->
+            false
+    end.
+
+last_consistent_replica(Cs, Tab, Downs) ->
     Storage = mnesia_lib:cs_to_storage_type(node(), Cs),
     Ram = Cs#cstruct.ram_copies,
     Disc = Cs#cstruct.disc_copies,
@@ -1775,7 +1789,7 @@ update_where_to_wlock(Tab) ->
 %% This code is rpc:call'ed from the tab_copier process
 %% when it has *not* released it's table lock
 unannounce_add_table_copy(Tab, To) ->
-    ?SAFE(del_active_replica(Tab, To)),
+    ?CATCH(del_active_replica(Tab, To)),
     try To = val({Tab , where_to_read}),
 	 mnesia_lib:set_remote_where_to_read(Tab)
     catch _:_ -> ignore
@@ -2054,7 +2068,7 @@ opt_start_sender2([Sender|R], Pids, Kept, LoaderQ) ->
 	    Pid = spawn_link(?MODULE, send_and_reply,[self(), Sender]),
 	    opt_start_sender2(R,[{Pid,Sender}|Pids],Kept,LoaderQ);
 	true ->
-	    verbose("Send table failed ~p not active on this node ~n", [Tab]),
+	    verbose("Send table failed ~tp not active on this node ~n", [Tab]),
 	    Sender#send_table.receiver_pid ! {copier_done, node()},
 	    opt_start_sender2(R,Pids, Kept, LoaderQ)
     end.
@@ -2101,10 +2115,17 @@ already_loading2(Tab, [{_,#disc_load{table=Tab}}|_]) -> true;
 already_loading2(Tab, [_|Rest]) -> already_loading2(Tab,Rest);
 already_loading2(_,[]) -> false.
 
-start_remote_sender(Node, Tab, Receiver, Storage) ->
-    Msg = #send_table{table = Tab,
-		      receiver_pid = Receiver,
-		      remote_storage = Storage},
+start_remote_sender(Node, Tab, Receiver, Storage, Why) ->
+    Msg = case ?catch_val({protocol, Node}) of
+              {Ver, _} when Ver < {8,6} ->
+                  {send_table, Tab, Receiver, Storage};
+              _ ->
+                  #send_table{table = Tab,
+                              receiver_pid = Receiver,
+                              remote_storage = Storage,
+                              reason = Why
+                             }
+          end,
     gen_server:cast({?SERVER_NAME, Node}, Msg).
 
 dump_and_reply(ReplyTo, Worker) ->
@@ -2120,13 +2141,15 @@ dump_and_reply(ReplyTo, Worker) ->
     unlink(ReplyTo),
     exit(normal).
 
-send_and_reply(ReplyTo, Worker) ->
+send_and_reply(ReplyTo, #send_table{table=Tab, remote_storage=Storage, receiver_pid=Pid, reason=Reason}) ->
     %% No trap_exit, die intentionally instead
-    Res = mnesia_loader:send_table(Worker#send_table.receiver_pid,
-				   Worker#send_table.table,
-				   Worker#send_table.remote_storage),
-    ReplyTo ! #sender_done{worker_pid = self(),
-			   worker_res = Res},
+    Res = mnesia_loader:send_table(Pid, Tab, Storage, Reason),
+    case Res of
+        {error, {no_exists, _}} ->
+            Pid ! {copier_done, node()};
+        _ -> ok
+    end,
+    ReplyTo ! #sender_done{worker_pid = self(), worker_res = Res},
     unlink(ReplyTo),
     exit(normal).
 
@@ -2161,6 +2184,15 @@ load_table_fun(#net_load{cstruct=Cs, table=Tab, reason=Reason, opt_reply_to=Repl
 		       {dumper,{add_table_copy,_}} -> true;
 		       _ -> false
 		   end,
+
+    OnlyRamCopies = case Cs of
+                        #cstruct{disc_copies = DC,
+                                 disc_only_copies = DOC,
+                                 external_copies = Ext} ->
+                            [] =:= (DC ++ (DOC ++ Ext)) -- [node()];
+                        _ ->
+                            false
+                    end,
     if
 	ReadNode == node() ->
 	    %% Already loaded locally
@@ -2172,6 +2204,8 @@ load_table_fun(#net_load{cstruct=Cs, table=Tab, reason=Reason, opt_reply_to=Repl
 	    end;
 	AccessMode == read_only, not AddTableCopy ->
 	    fun() -> disc_load_table(Tab, Reason, ReplyTo) end;
+        Active =:= [], AddTableCopy, OnlyRamCopies ->
+            fun() -> disc_load_table(Tab, Reason, ReplyTo) end;
 	true ->
 	    fun() ->
 		    %% Either we cannot read the table yet
@@ -2239,7 +2273,7 @@ disc_load_table(Tab, Reason, ReplyTo) ->
 	    Done#loader_done{is_loaded = false,
 			     reply = Res};
 	true ->
-	    fatal("Cannot load table ~p from disc: ~p~n", [Tab, Res])
+	    fatal("Cannot load table ~tp from disc: ~tp~n", [Tab, Res])
     end.
 
 filter_active(Tab) ->
