@@ -1,7 +1,7 @@
 %%
 %% %CopyrightBegin%
 %%
-%% Copyright Ericsson AB 2011-2017. All Rights Reserved.
+%% Copyright Ericsson AB 2011-2021. All Rights Reserved.
 %%
 %% Licensed under the Apache License, Version 2.0 (the "License");
 %% you may not use this file except in compliance with the License.
@@ -27,7 +27,7 @@
 	 handle_event/2, handle_cast/2]).
 
 -include_lib("wx/include/wx.hrl").
--include("../include/etop.hrl").
+-include("etop.hrl").
 -include("observer_defs.hrl").
 -include("etop_defs.hrl").
 
@@ -50,6 +50,7 @@
 -define(ID_TRACE_NEW, 208).
 -define(ID_TRACE_ALL, 209).
 -define(ID_ACCUMULATE, 210).
+-define(ID_GARBAGE_COLLECT, 211).
 
 -define(TRACE_PIDS_STR, "Trace selected process identifiers").
 -define(TRACE_NAMES_STR, "Trace selected processes, "
@@ -93,7 +94,7 @@ start_link(Notebook, Parent, Config) ->
 
 %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
 init([Notebook, Parent, Config]) ->
-    Attrs = observer_lib:create_attrs(),
+    Attrs = observer_lib:create_attrs(Notebook),
     Self = self(),
     Acc = maps:get(acc, Config, false),
     Holder = spawn_link(fun() -> init_table_holder(Self, Acc, Attrs) end),
@@ -147,11 +148,11 @@ create_list_box(Panel, Holder) ->
     ListCtrl = wxListCtrl:new(Panel, [{style, Style},
 				      {onGetItemText,
 				       fun(_, Row, Col) ->
-					       call(Holder, {get_row, self(), Row, Col})
+					       safe_call(Holder, {get_row, self(), Row, Col})
 				       end},
 				      {onGetItemAttr,
 				       fun(_, Item) ->
-					       call(Holder, {get_attr, self(), Item})
+					       safe_call(Holder, {get_attr, self(), Item})
 				       end}
 				     ]),
     Li = wxListItem:new(),
@@ -162,13 +163,14 @@ create_list_box(Panel, Holder) ->
 			   wxListCtrl:setColumnWidth(ListCtrl, Col, DefSize),
 			   Col + 1
 		   end,
-    ListItems = [{"Pid", ?wxLIST_FORMAT_CENTRE,  120},
-		 {"Name or Initial Func", ?wxLIST_FORMAT_LEFT, 200},
-%%		 {"Time", ?wxLIST_FORMAT_CENTRE, 50},
-		 {"Reds", ?wxLIST_FORMAT_RIGHT, 100},
-		 {"Memory", ?wxLIST_FORMAT_RIGHT, 100},
-		 {"MsgQ",  ?wxLIST_FORMAT_RIGHT, 50},
-		 {"Current Function", ?wxLIST_FORMAT_LEFT,  200}],
+    Scale = observer_wx:get_scale(),
+    ListItems = [{"Pid", ?wxLIST_FORMAT_CENTRE,  Scale*120},
+		 {"Name or Initial Func", ?wxLIST_FORMAT_LEFT, Scale*200},
+%%		 {"Time", ?wxLIST_FORMAT_CENTRE, Scale*50},
+		 {"Reds", ?wxLIST_FORMAT_RIGHT, Scale*100},
+		 {"Memory", ?wxLIST_FORMAT_RIGHT, Scale*100},
+		 {"MsgQ",  ?wxLIST_FORMAT_RIGHT, Scale*50},
+		 {"Current Function", ?wxLIST_FORMAT_LEFT,  Scale*200}],
     lists:foldl(AddListEntry, 0, ListItems),
     wxListItem:destroy(Li),
 
@@ -208,17 +210,26 @@ start_procinfo(Pid, Frame, Opened) ->
 	    Opened
     end.
 
+
+safe_call(Holder, What) ->
+    case call(Holder, What, 2000) of
+        Res when is_atom(Res) -> "";
+        Res -> Res
+    end.
+
 call(Holder, What) ->
+    call(Holder, What, infinity).
+
+call(Holder, What, TMO) ->
     Ref = erlang:monitor(process, Holder),
     Holder ! What,
     receive
-	{'DOWN', Ref, _, _, _} -> "";
+	{'DOWN', Ref, _, _, _} -> holder_dead;
 	{Holder, Res} ->
 	    erlang:demonitor(Ref),
 	    Res
-    after 2000 ->
-	    io:format("Hanging call ~tp~n",[What]),
-	    ""
+    after TMO ->
+	    timeout
     end.
 
 %%%%%%%%%%%%%%%%%%%%%%% Callbacks %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
@@ -269,7 +280,10 @@ code_change(_, _, State) ->
 
 handle_call(get_config, _, #state{holder=Holder, timer=Timer}=State) ->
     Conf = observer_lib:timer_config(Timer),
-    Accum = call(Holder, {get_accum, self()}),
+    Accum = case safe_call(Holder, {get_accum, self()}) of
+                Bool when is_boolean(Bool) -> Bool;
+                _ -> false
+            end,
     {reply, Conf#{acc=>Accum}, State};
 
 handle_call(Msg, _From, State) ->
@@ -316,6 +330,9 @@ handle_event(#wx{id=?ID_KILL}, #state{right_clicked_pid=Pid, sel=Sel0}=State) ->
     Sel = rm_selected(Pid,Sel0),
     {noreply, State#state{sel=Sel}};
 
+handle_event(#wx{id=?ID_GARBAGE_COLLECT}, #state{sel={_, Pids}}=State) ->
+    _ = [rpc:call(node(Pid), erlang, garbage_collect, [Pid]) || Pid <- Pids],
+    {noreply, State};
 
 handle_event(#wx{id=?ID_PROC},
 	     #state{panel=Panel, right_clicked_pid=Pid, procinfo_menu_pids=Opened}=State) ->
@@ -370,6 +387,7 @@ handle_event(#wx{event=#wxList{type=command_list_item_right_click,
 		wxMenu:append(Menu, ?ID_TRACE_NAMES,
 			      "Trace selected processes by name (all nodes)",
 			      [{help, ?TRACE_NAMES_STR}]),
+		wxMenu:append(Menu, ?ID_GARBAGE_COLLECT, "Garbage collect processes"),
 		wxMenu:append(Menu, ?ID_KILL, "Kill process " ++ pid_to_list(P)),
 		wxWindow:popupMenu(Panel, Menu),
 		wxMenu:destroy(Menu),
@@ -465,7 +483,7 @@ init_table_holder(Parent, Accum0, Attrs) ->
     Backend = spawn_link(node(), observer_backend, procs_info, [self()]),
     Accum = case Accum0 of
                 true -> true;
-                false -> []
+                _ -> []
             end,
     table_holder(#holder{parent=Parent,
 			 info=array:new(),
@@ -572,7 +590,8 @@ change_accum(true, S0) ->
     S0#holder{accum=true};
 change_accum(false, S0=#holder{info=Info}) ->
     self() ! refresh,
-    S0#holder{accum=lists:sort(array:to_list(Info))}.
+    Accum = [{Pid, Reds} || #etop_proc_info{pid=Pid, reds=Reds} <- array:to_list(Info)],
+    S0#holder{accum=lists:sort(Accum)}.
 
 handle_update_old(#etop_info{procinfo=ProcInfo0},
                   S0=#holder{parent=Parent, sort=Sort=#sort{sort_key=KeyField}}) ->
